function y=MGPCall(b,Alist,Plist,Rlist,nu1,nu2,mu,Smoother,Llist,Smoother2,Llist2)
% y=MGPCall(b,Alist,Plist,Rlist,nu1,nu2,mu,Smoother,Llist)
%
% recursive multigrid call as PRECONDITIONER.
% The preconditioner M_n satisfies 
% I-M_n A=(I-S_{2,n}A_n)^nu2(I-P_nM_{n-1}R_nA_n)^mu(I-S_{1,n}A_n)^nu1,
% where A_n is the initial system, S_{1,n},S_{2,n} correspond to the smoothers,
% P_n,R_n are prolongation/restriction and M_{n-1} is the recursive multilevel
% call
%
% --------------------------------------------------------------------------
%
% Input
% -----
% b         right hand side on the finest level
% Alist     cell array of system matrices for any level
% Plist     cell array of prolongation operators
% Rlist     cell array of restriction  operators (transposed!)
% nu1       number of pre-smoothing steps
% nu2       number of post-smoothing steps
% Smoother  function name of the Smoother
% Llist     parameter list for the smoother. The smoother is called by
%           feval(Smoother,b,x,Llist)
% Smoother2 OPTIONAL, if nu1>0 and nu2>0, Smoother refers to pre-smoothing
%           Smoother2 refers to post-smoothing
% Llist2    OPTIONAL, parameter list for the post-smoother. The post-smoother
%           is called by feval(Smoother2,b,x,Llist2)
% 
% --------------------------------------------------------------------------
%
% Output
% ------
% y         approximate solution obtained by one multigrid preconditioning step


% for an iterative scheme with error propagation matrix 
% (I-A_k A)*...(I-A_1 A) =: I-MA we have an implicit definition of M via
% M = [I-(I-A_k A)*...(I-A_1 A)]A^{-1} where A^{-1} never has to be applied
% since
% M==M_{1,k}
% =[I-(I-A_k A)*...(I-A_1 A)]A^{-1}
% =[I-(I-A_k A)*...(I-A_2 A)]A^{-1}(I-AA_1) +A_1
% ==M_{2,k}(I-AA_1) +A_1
% From this formula we obtain a successive application rule for the operator M.

m=length(Alist);
y=zeros(size(b,1),size(b,2));
r=y;
d=y;

if m==1
   % direct solution on the coarsest grid
   y=(Llist{m}.U)\ ((Llist{m}.L)\b);
else   

   % pre-smoothing
   for l=1:nu1
       if l>1
	  b=b-Alist{1}*d;
       end % 
       d=feval(Smoother,b,r,Llist{1});
       y=y+d;
   end % for
 
   % coarse grid correction  
   v=2:m-1;
   B=zeros(size(Alist{2},1),size(b,2));
   if nargin==11
      for l=1:mu
	  if l>1 | nu1>0
	     b=b-Alist{1}*d;
	  end % 
	  B=Rlist{1}'*b;
	  d=Plist{1}*MGPCall(B,Alist([v m]),Plist(v),Rlist(v),nu1,nu2,mu,...
	                     Smoother,Llist([v m]),Smoother2,Llist2([v m]));
          y=y+d;
      end % for l
      % post-smoothing
      for l=1:nu2
	  if l>1 | nu1>0 | mu>0
	     b=b-Alist{1}*d;
	  end % 
	  d=feval(Smoother2,b,r,Llist2{1});
	  y=y+d;
      end % for
   else   
      for l=1:mu
	  if l>1 | nu1>0
	     b=b-Alist{1}*d;
	  end % 
	  B=Rlist{1}'*b;
	  d=Plist{1}*MGPCall(B,Alist([v m]),Plist(v),Rlist(v),nu1,nu2,mu,...
	                     Smoother,Llist([v m]));
          y=y+d;
      end % for l
      % post-smoothing
      for l=1:nu2
	  % this if-statement only makes sense for nu1=mu=0,nu2>0,l=1
	  if l>1 | nu1>0 | mu>0
	     b=b-Alist{1}*d;
	  end % 
	  d=feval(Smoother,b,r,Llist{1});
	  y=y+d;
      end % for
   end % if-else
         
end % if-else
